# Edge Case Analysis for EQFE Framework
#
# This script simulates various edge cases for the Environmental Quantum Field Effects
# framework to validate theoretical predictions under extreme parameter regimes.

import numpy as np
import matplotlib.pyplot as plt
from simulations.core.field_simulator import FieldSimulator
from simulations.core.quantum_correlations import CHSHCorrelation
from natural_systems.cellular_field_coupling import CellularFieldModel
import config

# Configure plots
plt.style.use('seaborn-v0_8-whitegrid')
plt.rcParams.update({
    'font.size': 12,
    'figure.figsize': (12, 8),
    'lines.linewidth': 2,
    'axes.grid': True,
    'grid.alpha': 0.3,
})

def run_edge_case_simulations():
    """
    Run simulations for edge cases to test the limits of the EQFE framework:
    1. Markovian limit: C(τ) ∝ δ(τ)
    2. White noise limit: Constant spectrum
    3. Zero temperature limit
    4. High temperature limit
    """
    print("Running edge case simulations for the EQFE framework...")
    
    # Create base simulator objects
    field_sim = FieldSimulator()
    corr_sim = CHSHCorrelation()
    
    # Time parameters
    t_values = np.linspace(0, 5.0, 100)  # time units normalized to correlation time
    
    # Prepare plot
    fig, axes = plt.subplots(2, 2, figsize=(14, 10))
    fig.suptitle("Edge Case Analysis for EQFE Framework", fontsize=16)
    
    # 1. Markovian Limit Simulation
    print("Simulating Markovian limit (delta correlation)...")
    axes[0, 0].set_title("Markovian Limit: C(τ) ∝ δ(τ)")
    axes[0, 0].set_xlabel("Time (norm. to correlation time)")
    axes[0, 0].set_ylabel("CHSH Parameter S(t)")
    
    # Setup Markovian correlation function (delta function approximation)
    field_sim.set_correlation_function(
        lambda tau: 1.0 * np.exp(-100.0 * np.abs(tau))  # Very sharp decay
    )
    
    # Run simulation for different coupling strengths
    coupling_values = [0.01, 0.05, 0.1, 0.2]
    for g in coupling_values:
        field_sim.set_coupling(g)
        S_values = []
        for t in t_values:
            # In Markovian limit, we expect pure decoherence
            state = field_sim.evolve_state(t)
            S = corr_sim.calculate_CHSH(state)
            S_values.append(S)
        axes[0, 0].plot(t_values, S_values, label=f"g = {g}")
    
    axes[0, 0].axhline(y=2, color='r', linestyle='--', label="Classical bound")
    axes[0, 0].axhline(y=2*np.sqrt(2), color='g', linestyle='--', label="Quantum bound")
    axes[0, 0].legend()
    
    # 2. White Noise Limit Simulation
    print("Simulating white noise limit (constant spectrum)...")
    axes[0, 1].set_title("White Noise: Constant Spectrum")
    axes[0, 1].set_xlabel("Time (norm. to correlation time)")
    axes[0, 1].set_ylabel("CHSH Parameter S(t)")
    
    # Setup white noise correlation function
    field_sim.set_correlation_function(
        lambda tau: 1.0  # Constant correlation (white noise)
    )
    
    for g in coupling_values:
        field_sim.set_coupling(g)
        S_values = []
        for t in t_values:
            state = field_sim.evolve_state(t)
            S = corr_sim.calculate_CHSH(state)
            S_values.append(S)
        axes[0, 1].plot(t_values, S_values, label=f"g = {g}")
    
    axes[0, 1].axhline(y=2, color='r', linestyle='--', label="Classical bound")
    axes[0, 1].axhline(y=2*np.sqrt(2), color='g', linestyle='--', label="Quantum bound")
    axes[0, 1].legend()
    
    # 3. Zero Temperature Limit
    print("Simulating zero temperature limit...")
    axes[1, 0].set_title("Zero Temperature Limit")
    axes[1, 0].set_xlabel("Time (norm. to correlation time)")
    axes[1, 0].set_ylabel("CHSH Parameter S(t)")
    
    # Setup zero temperature correlation function
    field_sim.set_correlation_function(
        lambda tau: np.cos(0.5 * np.abs(tau)) * np.exp(-0.1 * np.abs(tau))  # Zero T has oscillatory behavior
    )
    field_sim.set_field_variance(0.01)  # Very low field variance at T=0
    
    for g in coupling_values:
        field_sim.set_coupling(g)
        S_values = []
        for t in t_values:
            state = field_sim.evolve_state(t)
            S = corr_sim.calculate_CHSH(state)
            S_values.append(S)
        axes[1, 0].plot(t_values, S_values, label=f"g = {g}")
    
    axes[1, 0].axhline(y=2, color='r', linestyle='--', label="Classical bound")
    axes[1, 0].axhline(y=2*np.sqrt(2), color='g', linestyle='--', label="Quantum bound")
    axes[1, 0].legend()
    
    # 4. High Temperature Limit
    print("Simulating high temperature limit...")
    axes[1, 1].set_title("High Temperature Limit")
    axes[1, 1].set_xlabel("Time (norm. to correlation time)")
    axes[1, 1].set_ylabel("CHSH Parameter S(t)")
    
    # Setup high temperature correlation function
    field_sim.set_correlation_function(
        lambda tau: np.exp(-0.5 * np.abs(tau))  # Fast decay typical at high T
    )
    field_sim.set_field_variance(10.0)  # High field variance at high T
    
    for g in coupling_values:
        field_sim.set_coupling(g)
        S_values = []
        for t in t_values:
            state = field_sim.evolve_state(t)
            S = corr_sim.calculate_CHSH(state)
            S_values.append(S)
        axes[1, 1].plot(t_values, S_values, label=f"g = {g}")
    
    axes[1, 1].axhline(y=2, color='r', linestyle='--', label="Classical bound")
    axes[1, 1].axhline(y=2*np.sqrt(2), color='g', linestyle='--', label="Quantum bound")
    axes[1, 1].legend()
    
    # Adjust layout and save
    plt.tight_layout(rect=[0, 0, 1, 0.95])
    plt.savefig('simulations/analysis/edge_case_results.png', dpi=300)
    plt.show()
    
    print("Edge case analysis complete. Results saved to 'simulations/analysis/edge_case_results.png'")
    
    # Return comparison with standard quantum optics predictions
    return compare_with_standard_theory()

def compare_with_standard_theory():
    """Compare EQFE results with standard quantum optics predictions"""
    
    # Standard theory predicts monotonic decoherence
    # EQFE predicts non-monotonic behavior under specific conditions
    
    # Time parameters 
    t_values = np.linspace(0, 5.0, 100)
    
    # Prepare plot
    plt.figure(figsize=(10, 6))
    plt.title("EQFE vs. Standard Quantum Optics Predictions", fontsize=14)
    plt.xlabel("Time (norm. to correlation time)")
    plt.ylabel("CHSH Parameter S(t)")
    
    # Setup simulator objects
    field_sim = FieldSimulator()
    corr_sim = CHSHCorrelation()
    
    # Standard quantum optics prediction (always monotonic decoherence)
    std_theory = lambda t, rate: 2 * np.sqrt(2) * np.exp(-rate * t)
    plt.plot(t_values, [std_theory(t, 0.5) for t in t_values], 
             'b--', label="Standard QO (γ=0.5)")
    plt.plot(t_values, [std_theory(t, 1.0) for t in t_values], 
             'g--', label="Standard QO (γ=1.0)")
    
    # EQFE prediction (can show enhancement under right conditions)
    field_sim.set_correlation_function(
        lambda tau: np.cos(2.0 * np.abs(tau)) * np.exp(-0.4 * np.abs(tau))
    )
    field_sim.set_field_variance(1.0)
    field_sim.set_coupling(0.1)
    
    S_values = []
    for t in t_values:
        state = field_sim.evolve_state(t)
        S = corr_sim.calculate_CHSH(state)
        S_values.append(S)
    plt.plot(t_values, S_values, 'r-', label="EQFE prediction")
    
    # Reference lines
    plt.axhline(y=2, color='k', linestyle=':', label="Classical bound")
    plt.axhline(y=2*np.sqrt(2), color='k', linestyle='--', label="Quantum bound")
    
    plt.legend()
    plt.grid(True, alpha=0.3)
    plt.tight_layout()
    plt.savefig('simulations/analysis/theory_comparison.png', dpi=300)
    
    print("Theory comparison complete. Results saved to 'simulations/analysis/theory_comparison.png'")
    return {
        "conclusion": "EQFE framework predicts non-monotonic behavior with possible " + 
                     "enhancement under specific environmental conditions, while " + 
                     "standard quantum optics always predicts monotonic decoherence."
    }

if __name__ == "__main__":
    results = run_edge_case_simulations()
    print("\nConclusion:")
    print(results["conclusion"])
